"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transform = transform;

function _sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return _sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }

// FIXME(sven): do the same with all block instructions, must be more generic here
var _require = require("../../traverse"),
    traverse = _require.traverse;

var t = require("../../index");

function transform(ast) {
  var functionsInProgram = [];
  var globalsInProgram = []; // First collect the indices of all the functions in the Program

  traverse(ast, {
    ModuleImport: function (_ModuleImport) {
      function ModuleImport(_x) {
        return _ModuleImport.apply(this, arguments);
      }

      ModuleImport.toString = function () {
        return _ModuleImport.toString();
      };

      return ModuleImport;
    }(function (_ref) {
      var node = _ref.node;
      functionsInProgram.push(t.identifier(node.name));
    }),
    Global: function (_Global) {
      function Global(_x2) {
        return _Global.apply(this, arguments);
      }

      Global.toString = function () {
        return _Global.toString();
      };

      return Global;
    }(function (_ref2) {
      var node = _ref2.node;

      if (node.name != null) {
        globalsInProgram.push(node.name);
      }
    }),
    Func: function (_Func) {
      function Func(_x3) {
        return _Func.apply(this, arguments);
      }

      Func.toString = function () {
        return _Func.toString();
      };

      return Func;
    }(function (_ref3) {
      var node = _ref3.node;

      if (node.name == null) {
        return;
      }

      functionsInProgram.push(node.name);
    })
  }); // Transform the actual instruction in function bodies

  traverse(ast, {
    Func: function (_Func2) {
      function Func(_x4) {
        return _Func2.apply(this, arguments);
      }

      Func.toString = function () {
        return _Func2.toString();
      };

      return Func;
    }(function (path) {
      transformFuncPath(path, functionsInProgram, globalsInProgram);
    })
  });
}

function transformFuncPath(funcPath, functionsInProgram, globalsInProgram) {
  var funcNode = funcPath.node;
  var params = funcNode.params;
  traverse(funcNode, {
    Instr: function Instr(instrPath) {
      var instrNode = instrPath.node;

      if (instrNode.id === "get_local" || instrNode.id === "set_local" || instrNode.id === "tee_local") {
        var _instrNode$args = _slicedToArray(instrNode.args, 1),
            firstArg = _instrNode$args[0];

        if (firstArg.type === "Identifier") {
          var offsetInParams = params.findIndex(function (_ref4) {
            var id = _ref4.id;
            return id === firstArg.value;
          });

          if (offsetInParams === -1) {
            throw new Error("".concat(firstArg.value, " not found in ").concat(instrNode.id, ": not declared in func params"));
          }

          var indexNode = t.indexLiteral(offsetInParams); // Replace the Identifer node by our new NumberLiteral node

          instrNode.args[0] = indexNode;
        }
      }

      if (instrNode.id === "get_global" || instrNode.id === "set_global") {
        var _instrNode$args2 = _slicedToArray(instrNode.args, 1),
            _firstArg = _instrNode$args2[0];

        if (_firstArg.type === "Identifier") {
          var offsetInGlobalsInProgram = globalsInProgram.findIndex(function (_ref5) {
            var value = _ref5.value;
            return value === _firstArg.value;
          });

          if (offsetInGlobalsInProgram === -1) {
            throw new Error("global ".concat(_firstArg.value, " not found in module"));
          }

          var _indexNode = t.indexLiteral(offsetInGlobalsInProgram); // Replace the Identifer node by our new NumberLiteral node


          instrNode.args[0] = _indexNode;
        }
      }
    },
    CallInstruction: function (_CallInstruction) {
      function CallInstruction(_x5) {
        return _CallInstruction.apply(this, arguments);
      }

      CallInstruction.toString = function () {
        return _CallInstruction.toString();
      };

      return CallInstruction;
    }(function (_ref6) {
      var node = _ref6.node;
      var index = node.index;

      if (index.type === "Identifier") {
        var offsetInFunctionsInProgram = functionsInProgram.findIndex(function (_ref7) {
          var value = _ref7.value;
          return value === index.value;
        });

        if (offsetInFunctionsInProgram === -1) {
          throw new Error("".concat(index.value, " not found in CallInstruction: not declared in Program"));
        }

        var indexNode = t.indexLiteral(offsetInFunctionsInProgram); // Replace the index Identifier

        node.index = indexNode;
      }
    })
  });
}