"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = validate;

var _require = require("../AST/traverse"),
    traverse = _require.traverse;

var _require2 = require("./type-inference"),
    getType = _require2.getType,
    typeEq = _require2.typeEq;

function validate(ast) {
  var errors = [];
  traverse(ast, {
    Func: function (_Func) {
      function Func(_x) {
        return _Func.apply(this, arguments);
      }

      Func.toString = function () {
        return _Func.toString();
      };

      return Func;
    }(function (_ref) {
      var node = _ref.node;
      // Since only one return is allowed at the moment, we don't need to check
      // them all.
      var resultType = node.result;
      var inferedResultType = getType(node.body); // Type is unknown, we can not verify the result type

      if (typeof inferedResultType === "undefined") {
        return;
      } // $FlowIgnore


      if (typeEq(resultType, inferedResultType) === false) {
        var name = "anonymous";

        if (node.name != null) {
          name = node.name.value;
        }

        errors.push("- Type mismatch: function '".concat(name, "' expected result type ").concat(JSON.stringify(resultType), ",") + " but ".concat(JSON.stringify(inferedResultType), " given."));
        return;
      }
    })
  });
  return errors;
}