"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createInstance = createInstance;

var _require = require("../../partial-evaluation"),
    evaluate = _require.evaluate;

var _require2 = require("../../../compiler/validation/is-const"),
    isConst = _require2.isConst;

var _require3 = require("../../../compiler/validation/type-inference"),
    getType = _require3.getType,
    typeEq = _require3.typeEq;

var _require4 = require("../../../errors"),
    CompileError = _require4.CompileError;

function createInstance(allocator, node) {
  var value;
  var _node$globalType = node.globalType,
      valtype = _node$globalType.valtype,
      mutability = _node$globalType.mutability;

  if (node.init.length > 0 && isConst(node.init) === false) {
    throw new CompileError("constant expression required");
  } // None or multiple constant expressions in the initializer seems not possible
  // TODO(sven): find a specification reference for that


  if (node.init.length > 1 || node.init.length === 0) {
    throw new CompileError("type mismatch");
  } // Validate the type


  var resultInferedType = getType(node.init);

  if (resultInferedType != null && typeEq([node.globalType.valtype], resultInferedType) === false) {
    throw new CompileError("type mismatch");
  }

  var res = evaluate(allocator, node.init);

  if (res != null) {
    value = res.value;
  }

  return {
    type: valtype,
    mutability: mutability,
    value: value
  };
}